import 'ckeditor5-build-classic-simple-upload-adapter-image-resize';
import Choices from "choices.js";
import Sortable from 'sortablejs';
import flatpickr from "flatpickr";
import bootstrap from "bootstrap/dist/js/bootstrap.bundle";
import Rails from "@rails/ujs";

import {Application} from "stimulus";
import {definitionsFromContext} from "stimulus/webpack-helpers";

export class Headmin {
    static start() {
        // Init Rails UJS
        Rails.start()

        // Init Stimulus
        const application = Application.start()
        const context = require.context("./controllers", true, /\.js$/)
        application.load(definitionsFromContext(context))

        // Init Plugins
        this.initPlugins()
    }

    static initPlugins() {
        this.initChoices()
        this.initFlatpickrs()
        this.initToasts()
        this.initPopovers()
        this.initCKEditors()
    }

    static initFlatpickrs() {
        document.querySelectorAll('.flatpickr').forEach((element) => {
            this.initFlatpickr(element)
        })
    }

    static initFlatpickr(element) {
        flatpickr(element, {
            allowInput: true,
            altInput: true,
            altFormat: 'd/m/Y',
        });
    }

    static initCKEditors() {
        document.querySelectorAll('.ckeditor').forEach((element) => {
            this.initCKEditor(element)
        })
    }

    static csrfToken() {
        return document.querySelector('meta[name="csrf-token"]').content
    }

    static initCKEditor(element) {
        const defaultToolbarItems = [
            'heading', '|', 'bold', 'italic', 'bulletedList', 'numberedList', '|', 'outdent',
            'indent', '|', 'link', 'imageUpload', 'blockQuote', 'insertTable', 'mediaEmbed', '|', 'undo', 'redo'
        ]
        const requestedToolbarItems = element.dataset.ckeditorToolbar ? JSON.parse(element.dataset.ckeditorToolbar) : undefined
        const toolbarItems = requestedToolbarItems ? requestedToolbarItems : defaultToolbarItems
        const uploadPath = element.dataset.ckeditorUploadPath

        ClassicEditor.create(
            element,
            {
                toolbar: {
                    items: toolbarItems
                },
                language: 'en',
                image: {
                    toolbar: [
                        'imageTextAlternative',
                        'imageStyle:full',
                        'imageStyle:side'
                    ]
                },
                table: {
                    contentToolbar: [
                        'tableColumn',
                        'tableRow',
                        'mergeTableCells'
                    ]
                },

                simpleUpload: {
                    uploadUrl: uploadPath,
                    withCredentials: true,
                    headers: {
                        'X-CSRF-TOKEN': this.csrfToken(),
                    }
                }

            })
    }

    static initChoices() {
        this.initChoicesSelectTags()
        this.initChoicesMultipleSelect()
    }

    static initChoicesSelectTags() {
        document.querySelectorAll('.select-tags').forEach((select) => {

            // Skip if already initialized
            if (select.dataset.choice === 'active') {
                return
            }

            // Create a new instance
            new Choices(select, {
                removeItemButton: true,
                loadingText: 'Loading...',
                noResultsText: 'No results found',
                noChoicesText: 'No choices to choose from',
                itemSelectText: 'Press to select',
                addItems: true,
                addItemText: (value) => {
                    return `Press Enter to add <b>"${value}"</b>`;
                },
                maxItemText: (maxItemCount) => {
                    return `Only ${maxItemCount} values can be added`;
                },
            });
        })
    }

    static initChoicesMultipleSelect() {
        document.querySelectorAll('.multiple-select').forEach((select) => {

            // Skip if already initialized
            if (select.dataset.choice === 'active') {
                return
            }

            // Create a new instance
            new Choices(select, {
                removeItemButton: true,
                loadingText: 'Loading...',
                noResultsText: 'No results found',
                noChoicesText: 'No choices to choose from',
                itemSelectText: 'Press to select',
                addItems: true,
                addItemText: (value) => {
                    return `Press Enter to add <b>"${value}"</b>`;
                },
                maxItemText: (maxItemCount) => {
                    return `Only ${maxItemCount} values can be added`;
                },
            });
        })
    }

    static initToasts() {
        document.querySelectorAll('.toast').forEach((toast) => {
            new bootstrap.Toast(toast, {})
        })
    }

    static initPopovers() {
        var popoverTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="popover"]'))
        var popoverList = popoverTriggerList.map(function (popoverTriggerEl) {
            return new bootstrap.Popover(popoverTriggerEl, {sanitize: false})
        })
    }
}